from json import JSONDecoder, JSONEncoder
from typing import IO, Any, Callable, NoReturn

def strict_constant(o: Any) -> NoReturn: ...
def dump(
    obj: Any,
    fp: IO[str],
    *,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = True,
    cls: type[JSONEncoder] | None = None,
    indent: int | str | None = None,
    separators: tuple[str, str] | None = None,
    default: Callable[[Any], Any] | None = None,
    sort_keys: bool = False,
    **kw: Any,
) -> None: ...
def dumps(
    obj: Any,
    *,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = True,
    cls: type[JSONEncoder] | None = None,
    indent: int | str | None = None,
    separators: tuple[str, str] | None = None,
    default: Callable[[Any], Any] | None = None,
    sort_keys: bool = False,
    **kw: Any,
) -> str: ...
def load(
    fp: IO[str],
    *,
    cls: type[JSONDecoder] | None = None,
    object_hook: Callable[[dict[str, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
    **kw: Any,
) -> Any: ...
def loads(
    s: str,
    *,
    cls: type[JSONDecoder] | None = None,
    object_hook: Callable[[dict[str, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
    **kw: Any,
) -> Any: ...



import sys
from collections.abc import Callable, Mapping, Sequence
from typing import Any, Literal, Protocol, TypeVar

from django.http import HttpRequest
from django.http.response import HttpResponseBase
from rest_framework.authentication import BaseAuthentication
from rest_framework.parsers import BaseParser
from rest_framework.permissions import _PermissionClass
from rest_framework.renderers import BaseRenderer
from rest_framework.request import Request
from rest_framework.schemas.inspectors import ViewInspector
from rest_framework.throttling import BaseThrottle
from rest_framework.views import APIView, AsView  # noqa: F401
from typing_extensions import Concatenate, ParamSpec, TypeAlias

_View = TypeVar("_View", bound=Callable[..., HttpResponseBase])
_P = ParamSpec("_P")
_RESP = TypeVar("_RESP", bound=HttpResponseBase)

_MixedCaseHttpMethod: TypeAlias = Literal[
    "GET",
    "POST",
    "DELETE",
    "PUT",
    "PATCH",
    "TRACE",
    "HEAD",
    "OPTIONS",
    "get",
    "post",
    "delete",
    "put",
    "patch",
    "trace",
    "head",
    "options",
]
if sys.version_info >= (3, 11):
    from http import HTTPMethod

    _HttpMethod: TypeAlias = _MixedCaseHttpMethod | HTTPMethod
else:
    _HttpMethod: TypeAlias = _MixedCaseHttpMethod

class MethodMapper(dict):
    def __init__(self, action: _View, methods: Sequence[str]) -> None: ...
    def _map(self, method: str, func: _View) -> _View: ...
    def get(self, func: _View) -> _View: ...  # type: ignore
    def post(self, func: _View) -> _View: ...
    def put(self, func: _View) -> _View: ...
    def patch(self, func: _View) -> _View: ...
    def delete(self, func: _View) -> _View: ...
    def head(self, func: _View) -> _View: ...
    def options(self, func: _View) -> _View: ...
    def trace(self, func: _View) -> _View: ...

class ViewSetAction(Protocol[_View]):
    detail: bool
    url_path: str
    url_name: str
    kwargs: Mapping[str, Any]
    mapping: MethodMapper
    __call__: _View
    __name__: str

def api_view(
    http_method_names: Sequence[str] | None = ...,
) -> Callable[[Callable[Concatenate[Request, _P], _RESP]], AsView[Callable[Concatenate[HttpRequest, _P], _RESP]]]: ...
def renderer_classes(renderer_classes: Sequence[BaseRenderer | type[BaseRenderer]]) -> Callable[[_View], _View]: ...
def parser_classes(parser_classes: Sequence[BaseParser | type[BaseParser]]) -> Callable[[_View], _View]: ...
def authentication_classes(
    authentication_classes: Sequence[BaseAuthentication | type[BaseAuthentication]],
) -> Callable[[_View], _View]: ...
def throttle_classes(throttle_classes: Sequence[BaseThrottle | type[BaseThrottle]]) -> Callable[[_View], _View]: ...
def permission_classes(permission_classes: Sequence[_PermissionClass]) -> Callable[[_View], _View]: ...
def schema(view_inspector: ViewInspector | type[ViewInspector] | None) -> Callable[[_View], _View]: ...
def action(
    methods: Sequence[_HttpMethod] | None = ...,
    detail: bool = ...,
    url_path: str | None = ...,
    url_name: str | None = ...,
    suffix: str | None = ...,
    name: str | None = ...,
    **kwargs: Any,
) -> Callable[[_View], ViewSetAction[_View]]: ...



from typing import Any

def manager_repr(value: Any) -> str: ...
def smart_repr(value: Any) -> str: ...
def field_repr(field: Any, force_many: bool = ...) -> str: ...
def serializer_repr(serializer: Any, indent: Any, force_many: Any | None = ...) -> str: ...
def list_repr(serializer: Any, indent: Any) -> str: ...



from typing import Any

from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.schemas import SchemaGenerator
from rest_framework.views import APIView

class SchemaView(APIView):
    _ignore_model_permissions: bool
    public: bool
    schema_generator: SchemaGenerator | None

    def get(self, request: Request, *args: Any, **kwargs: Any) -> Response: ...



from typing import Any

from django.db.models import QuerySet
from typing_extensions import TypeAlias

try:
    from django.contrib.postgres import fields as postgres_fields
except ImportError:
    postgres_fields: TypeAlias = None  # type: ignore[no-redef]
try:
    import coreapi  # type: ignore[import-untyped]
except ImportError:
    coreapi: TypeAlias = None  # type: ignore[no-redef]
try:
    import uritemplate
except ImportError:
    uritemplate: TypeAlias = None  # type: ignore[no-redef]
try:
    import coreschema  # type: ignore[import-untyped]
except ImportError:
    coreschema: TypeAlias = None  # type: ignore[no-redef]
try:
    import yaml
except ImportError:
    yaml: TypeAlias = None  # type: ignore[no-redef]
try:
    import inflection  # type: ignore[import-not-found,unused-ignore]
except ImportError:
    inflection: TypeAlias = None  # type: ignore[no-redef]
try:
    import requests
except ImportError:
    requests: TypeAlias = None  # type: ignore[no-redef]
try:
    import pygments
except ImportError:
    pygments: TypeAlias = None  # type: ignore[no-redef]

try:
    import markdown
    from markdown.preprocessors import Preprocessor
    def apply_markdown(text: str) -> str: ...

    class CodeBlockPreprocessor(Preprocessor):
        pattern: Any
        formatter: Any
        def run(self, lines: list[str]) -> list[str]: ...

except ImportError:
    apply_markdown: TypeAlias = None  # type: ignore[no-redef]
    markdown: TypeAlias = None  # type: ignore[no-redef]

def pygments_css(style: Any) -> str | None: ...
def pygments_highlight(text: str, lang: str, style: Any) -> Any: ...
def md_filter_add_syntax_highlight(md: Any) -> bool: ...
def unicode_http_header(value: str | bytes) -> str: ...

SHORT_SEPARATORS: tuple[str, str]
LONG_SEPARATORS: tuple[str, str]
INDENT_SEPARATORS: tuple[str, str]

__all__ = [
    "coreapi",
    "coreschema",
    "requests",
    "postgres_fields",
    "QuerySet",
    "uritemplate",
    "yaml",
    "inflection",
    "pygments",
    "markdown",
    "apply_markdown",
    "Preprocessor",
    "CodeBlockPreprocessor",
    "pygments_css",
    "pygments_highlight",
    "md_filter_add_syntax_highlight",
    "unicode_http_header",
    "SHORT_SEPARATORS",
    "LONG_SEPARATORS",
    "INDENT_SEPARATORS",
]
